/* @flow */
import * as React from 'react';
import PropTypes            from 'prop-types';
import invariant            from 'invariant';
import type Environment     from './Environment';
import type { AirfluxApi }  from './AirfluxApi';



/**
 * Creates an "airflux" application. What does it mean: the environment defined in this component will be passed in context
 * to all children component.
 * Meaning any ConnectStore you use in the hierarchy will automatically have access to the stores of the environment.
 * 
 */
export default class AirfluxApp extends React.Component< { environment: Environment; children: React.Node }, void > {
    getChildContext() : Object { 
        return { airflux: { environment: this.props.environment } }; 
    }
    
    static childContextTypes = { airflux: PropTypes.object };

    render() {
        return this.props.children;
    }
}
