/** @flow */
import Publisher from './Publisher';
import type Store from './Store';



/**
 * Fancy wrapper to define all stores of the application.
 * At this moment, it is used as an object.
 * There's more feature coming in the roadmap.
 * 
 * @example
 * class MyAppEnvironment extends Environment {
 *     sampleStore = new SampleStore();
 * }
 * 
 * @example
 * const myAppEnvironment = new MyAppEnvironment();
 * <airflux.AirfluxApp environment={ myAppEnvironment }>
 *     // your components here
 * </airflux.AirfluxApp>
 * 
 */
export default class Environment {
    $key: string;
    $value: Store< any >;

    constructor( stores: { [key: string ]: Store< any > } = {} ) {
        Object.keys( stores ).forEach( ( key ) => this[ key ] = stores[ key ] );
    }

    getPublishers(): { [key: string ]: Store< any > } {
        return Object.keys( this )
          .filter( key => typeof this[ key ].listen === 'function' )
          .map( key => ( { [key]: this[ key ] } ) )
          .reduce( ( a, b ) => Object.assign( {}, a, b ), {} );
    }
}
